>From 41a27a749392eb4b7774b7d4f14bceb5b6114f90 Mon Sep 17 00:00:00 2001
From: Roiy <roiy@cs.umanitoba.ca>
Date: Fri, 4 Oct 2013 13:59:21 -0500
Subject: [PATCH 7/9] Core control additions for MED

---
 include/vlc_demux.h        |   3 +
 include/vlc_input.h        |  80 ++++++
 src/input/control.c        | 700 ++++++++++++++++++++++++++++++++++++++++++++-
 src/input/event.c          |   5 +
 src/input/event.h          |   1 +
 src/input/input.c          |  77 ++++-
 src/input/input_internal.h |  16 ++
 src/input/var.c            |   2 +
 src/playlist/engine.c      |   1 +
 9 files changed, 879 insertions(+), 6 deletions(-)

diff --git a/include/vlc_demux.h b/include/vlc_demux.h
index 6cfca6b..a476487 100644
--- a/include/vlc_demux.h
+++ b/include/vlc_demux.h
@@ -167,6 +167,9 @@ enum demux_query_e
 
     DEMUX_CAN_SEEK,            /* arg1= bool*    can fail (assume false)*/
 
+    DEMUX_JUMP_SEGMENT,        /* arg1= char * , can fail */
+    DEMUX_SAVE_FILE,           /* arg1= char * , can fail . Saves file to filename arg1 */
+
     /* Navigation */
     DEMUX_NAV_ACTIVATE,        /* res=can fail */
     DEMUX_NAV_UP,              /* res=can fail */
diff --git a/include/vlc_input.h b/include/vlc_input.h
index 832c76c..9f3a30b 100644
--- a/include/vlc_input.h
+++ b/include/vlc_input.h
@@ -75,6 +75,57 @@ static inline seekpoint_t *vlc_seekpoint_Duplicate( const seekpoint_t *src )
 }
 
 /*****************************************************************************
+ * Seek Segment: Used in Meta layers
+ *****************************************************************************/
+struct seeksegment_t
+{
+    int64_t i_byte_offset;
+    int64_t i_time_offset;
+    int64_t i_byte_duration;
+    int64_t i_time_duration;
+    char    *psz_layerName;
+    int     eventNum;
+    char    *psz_text;
+    int     i_layer_id;
+};
+
+static inline seeksegment_t *vlc_seeksegment_New( void )    //ROIY - anyone use/should use this?
+{
+    seeksegment_t *segment = (seeksegment_t*)malloc( sizeof( seeksegment_t ) );
+    segment->i_byte_offset = -1;
+    segment->i_time_offset = -1;
+    segment->i_byte_duration = -1;
+    segment->i_time_duration = -1;
+    segment->psz_layerName = NULL;
+    segment->psz_text = NULL;
+    segment->eventNum = -1;
+    segment->i_layer_id = -1;
+    return segment;
+}
+
+static inline void vlc_seeksegment_Delete( seeksegment_t *segment )
+{
+    if( !segment ) return;
+    free( segment->psz_layerName );
+    free( segment->psz_text );
+    free( segment );
+}
+
+static inline seeksegment_t *vlc_seeksegment_Duplicate( const seeksegment_t *src )
+{
+    seeksegment_t *segment = vlc_seeksegment_New();
+    if( src->psz_layerName ) segment->psz_layerName = strdup( src->psz_layerName );
+    if( src->psz_text ) segment->psz_text = strdup( src->psz_text );
+    segment->i_time_offset = src->i_time_offset;
+    segment->i_byte_offset = src->i_byte_offset;
+    segment->i_time_duration = src->i_time_duration;
+    segment->i_byte_duration = src->i_byte_duration;
+    segment->eventNum = src->eventNum;
+    segment->i_layer_id = src->i_layer_id;
+    return segment;
+}
+
+/*****************************************************************************
  * Title:
  *****************************************************************************/
 typedef struct input_title_t
@@ -352,6 +403,9 @@ typedef enum input_event_type_e
     /* A chapter has been added or removed or selected. */
     INPUT_EVENT_CHAPTER,
 
+    /* Seeksegments have been updated */
+    INPUT_EVENT_SEEKSEGMENTS,
+
     /* A program ("program") has been added or removed or selected,
      * or "program-scrambled" has changed.*/
     INPUT_EVENT_PROGRAM,
@@ -432,6 +486,9 @@ enum input_query_e
     INPUT_NAV_LEFT,
     INPUT_NAV_RIGHT,
 
+    /* input variable "autoJump" */
+    INPUT_GET_AUTO_JUMP,             /* arg1= bool *      res=    */
+
     /* Meta datas */
     INPUT_ADD_INFO,   /* arg1= char* arg2= char* arg3=...     res=can fail */
     INPUT_REPLACE_INFOS,/* arg1= info_category_t *            res=cannot fail */
@@ -455,6 +512,27 @@ enum input_query_e
     /* titles */
     INPUT_GET_TITLE_INFO,     /* arg1=input_title_t** arg2= int * res=can fail */
 
+    /* Annotations & Segments */
+    INPUT_SET_SEGMENTS,    /* arg1= seeksegment_t ***, arg2= int *, res=can fail   */    
+    INPUT_SET_SEGMENT_AT_IDX,    /* arg1=seeksegment_t***, arg2=int, res=can fail */
+    INPUT_REPLACE_SEGMENT_AT_IDX,  /* arg1=seeksegment_t*, arg2=int Replaces event in index arg2 with event arg1. */
+    INPUT_DELETE_SEGMENT_AT_IDX,    /* arg1=int, arg2=int. Deletes event in index arg1, pushes all events backward (if arg2 is not -1, pushes all event numbers inthe arg2 layer.*/
+    INPUT_GET_SEGMENTS,    /* arg1=seeksegment_t***, arg2=int*, arg3=char*, res=can fail , if arg3 is null return all layers */
+    INPUT_GET_SELECTED_SEGMENTS,    /* arg1=seeksegment_t***, arg2=int*, res=can fail */
+    INPUT_GET_ANNOTATIONS,    /* arg1=seeksegment_t** returned event, int* event_index, int64_t queryTime, int layerId ; res=can fail, get annotation segment(s), if arg3 is -1 use current time, return segment if it matches arg4 layer, if arg4 is -1 return only if in selected segments */
+    INPUT_GET_NEXT_SEGMENT_IDX,    /* arg1=int*, arg2=int64_t, returns segment id at input time (current time if arg2 = -1) */
+    INPUT_GET_NEXT_SELECTED_SEGMENT,    /* arg1=seeksegment_t**, res=can fail, get next selected annotation segment */
+    INPUT_GET_PREV_SELECTED_SEGMENT,    /* arg1=seeksegment_t**, res=can fail, get previous selected annotation segment */
+    INPUT_GET_ANNOTATIONS_TIME,    /* arg1=char*, arg2=int64_t, arg3=int64_t* res=can fail. Gets elapsed segments time of layer [arg1]; if arg1 is NULL get data for all events. To get total segments time, mark arg2 as -1. */
+    INPUT_JUMP_SEGMENT,     /* const char *, res=can fail   */
+    INPUT_GET_LAYER_NAMES,  /* arg1=char****, arg2=int**, returns layers number and layers names array */
+    INPUT_GET_SELECTED_LAYERS, /* arg1=int***, arg2=int**, returns selected layers index numbers and size */
+    INPUT_SELECT_LAYER,    /* arg1=int, res=can fail. Selects and adds event layer [arg1] to player; if arg1 is -1 add all layers */
+    INPUT_DESELECT_LAYER, /* arg1=int, res=can fail. Deselects and removes event layer [arg1] from player; if arg1 is -1 remove all layers */
+    INPUT_ADD_LAYER,  /* arg1=char** */
+    INPUT_SAVE_ENHANCEMENT_FILE,    /* arg1=char*. Saves enhancement data file to arg1 */
+
+
     /* Attachments */
     INPUT_GET_ATTACHMENTS, /* arg1=input_attachment_t***, arg2=int*  res=can fail */
     INPUT_GET_ATTACHMENT,  /* arg1=input_attachment_t**, arg2=char*  res=can fail */
@@ -462,6 +540,8 @@ enum input_query_e
     /* On the fly input slave */
     INPUT_ADD_SLAVE,       /* arg1= const char * */
     INPUT_ADD_SUBTITLE,    /* arg1= const char *, arg2=bool b_check_extension */
+    INPUT_CHECK_SLAVE_EXISTS,  /* arg1= const char *, arg2=bool* exists */
+    INPUT_DEMUX_SLAVE,     /* arg1= char*, res=can fail ; Tries to set slave arg1 to demux */
 
     /* On the fly record while playing */
     INPUT_SET_RECORD_STATE, /* arg1=bool    res=can fail */
diff --git a/src/input/control.c b/src/input/control.c
index a4b2842..f48b1c7 100644
--- a/src/input/control.c
+++ b/src/input/control.c
@@ -30,14 +30,19 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
+#include <vlc_fs.h>
+#include <vlc_url.h>
 
 #include "input_internal.h"
 #include "event.h"
 #include "resource.h"
 #include "es_out.h"
+#include "demux.h"
 
 
 static void UpdateBookmarksOption( input_thread_t * );
+static int initEnhancementFile( input_thread_t *p_input );
+static int getSlaveId( input_thread_t *p_input, const char* psz_slave );
 
 /****************************************************************************
  * input_Control
@@ -66,12 +71,16 @@ int input_vaControl( input_thread_t *p_input, int i_query, va_list args )
     int i_bkmk = 0;
     int *pi_bkmk;
 
-    int i_int, *pi_int;
+    seeksegment_t *p_segment, ***p_segments;
+    int *pi_seeksegments;
+
+    int i_int, *pi_int, **ppi_int;
     bool b_bool, *pb_bool;
     double f, *pf;
     int64_t i_64, *pi_64;
 
     char *psz;
+    char ****psz_array;
     vlc_value_t val;
 
     switch( i_query )
@@ -97,7 +106,11 @@ int input_vaControl( input_thread_t *p_input, int i_query, va_list args )
 
         case INPUT_SET_TIME:
             i_64 = (int64_t)va_arg( args, int64_t );
-            return var_SetTime( p_input, "time", i_64 );
+            val.i_int = i_64;
+            input_ControlPush( p_input, INPUT_CONTROL_SET_TIME, &val );
+            var_SetTime( p_input, "time", i_64 );
+            return VLC_SUCCESS;
+            
 
         case INPUT_GET_RATE:
             pi_int = (int*)va_arg( args, int * );
@@ -136,6 +149,11 @@ int input_vaControl( input_thread_t *p_input, int i_query, va_list args )
             i_64 = (int64_t)va_arg( args, int64_t );
             return var_SetTime( p_input, "spu-delay", i_64 );
 
+        case INPUT_GET_AUTO_JUMP:
+            pb_bool = (bool*)va_arg( args, bool * );
+            *pb_bool = var_GetBool( p_input, "autoJump" );
+            return VLC_SUCCESS;
+
         case INPUT_NAV_ACTIVATE:
         case INPUT_NAV_UP:
         case INPUT_NAV_DOWN:
@@ -145,6 +163,50 @@ int input_vaControl( input_thread_t *p_input, int i_query, va_list args )
                                + INPUT_CONTROL_NAV_ACTIVATE, NULL );
             return VLC_SUCCESS;
 
+        case INPUT_JUMP_SEGMENT:
+        {
+            char *psz_direction = (char *)va_arg( args, char * );
+
+            msg_Err( p_input, "ROIY" );
+
+            //make sure there is med type slave
+            int i = getSlaveId( p_input, "med" );
+            
+            if ( i == -1 )
+                return VLC_SUCCESS; //no slaves -> no segments to jump to
+
+            if( demux_Control( p_input->p->slave[i]->p_demux, DEMUX_JUMP_SEGMENT,
+                            psz_direction ) != VLC_SUCCESS )
+            {
+                msg_Err( p_input, "Could not jump to %s segment", psz_direction );
+                return VLC_EGENERIC;
+            }
+
+            return VLC_SUCCESS;
+        }
+
+        case INPUT_DEMUX_SLAVE:     /* arg1=char*, res=can fail ; Tries to set slave arg1 to demux */
+        {
+            char *psz_slave = (char *)va_arg( args, char * );
+
+            int i;
+            //make sure there is correct type slave
+            i = getSlaveId( p_input, psz_slave );
+            if ( i == -1 )
+            {
+                msg_Err( p_input, "Slave %s not found", psz_slave );
+                return VLC_EGENERIC;
+            }
+
+            if( demux_Demux( p_input->p->slave[i]->p_demux ) < VLC_SUCCESS )
+            {
+                msg_Err( p_input, "Could not set slave %s to demux", psz_slave );
+                return VLC_EGENERIC;
+            }
+
+            return VLC_SUCCESS;
+        }
+
         case INPUT_ADD_INFO:
         {
             char *psz_cat = (char *)va_arg( args, char * );
@@ -302,6 +364,590 @@ int input_vaControl( input_thread_t *p_input, int i_query, va_list args )
             }
             break;
 
+        case INPUT_SET_SEGMENTS:    /* arg1=seeksegment_t***, arg2=int*, res=can fail */
+            p_segment = (seeksegment_t *)va_arg( args, seeksegment_t * );
+            pi_seeksegments = (int*)va_arg( args, int * );
+
+            vlc_mutex_lock( &p_input->p->p_item->lock );
+
+
+            if (p_segment && pi_seeksegments)
+            {
+                p_input->p->input.i_seeksegments = *pi_seeksegments;
+                p_input->p->input.seeksegments = realloc( p_input->p->input.seeksegments,
+                                              sizeof(seeksegment_t) * (*pi_seeksegments) ) ;   //ROIY - remove and fix this... ||| to create all at once?
+                p_input->p->input.seeksegments[*pi_seeksegments - 1] = vlc_seeksegment_Duplicate( p_segment );
+            }
+            else
+            {
+                vlc_mutex_unlock( &p_input->p->p_item->lock );
+                return VLC_EGENERIC;
+            }
+
+            vlc_mutex_unlock( &p_input->p->p_item->lock );
+            return VLC_SUCCESS;
+
+        case INPUT_SET_SEGMENT_AT_IDX:    /* arg1=seeksegment_t*, arg2=int, arg3=int. Sets arg1 in index arg2, pushes all events forward (if arg3 is not -1, pushes all event numbers inthe arg3 layer.*/
+        {
+            p_segment = (seeksegment_t *)va_arg( args, seeksegment_t * );
+            i_int = (int)va_arg( args, int );
+            int layerId = (int)va_arg( args, int );
+            int tempId = 0;
+
+            vlc_mutex_lock( &p_input->p->p_item->lock );
+
+
+            if ( p_segment && ( i_int >= 0 ) && ( i_int <= p_input->p->input.i_seeksegments ) )
+            {
+                p_input->p->input.i_seeksegments++;
+
+                p_input->p->input.seeksegments = realloc( p_input->p->input.seeksegments,
+                                              sizeof(seeksegment_t) * (p_input->p->input.i_seeksegments) ) ;
+
+                int i;
+                for ( i = p_input->p->input.i_seeksegments - 1; i > i_int; i-- )    //Start moving segments to the next index, going back until target
+                {
+                    p_input->p->input.seeksegments[i] = p_input->p->input.seeksegments[i - 1];
+
+                    if ( p_input->p->input.seeksegments[i]->i_layer_id == layerId )     //Advances events numbers
+                        tempId = p_input->p->input.seeksegments[i]->eventNum++;
+                        
+                }
+                p_input->p->input.seeksegments[i_int] = vlc_seeksegment_Duplicate( p_segment );
+
+                if ( !tempId && ( layerId > -1 ) )  //check what is the last event number for layer
+                {
+                    tempId = 1;
+
+                    for ( ; i >= 0; i-- )
+                        if ( p_input->p->input.seeksegments[i]->i_layer_id == layerId )
+                        {
+                            tempId = p_input->p->input.seeksegments[i]->eventNum + 1;
+                            break;
+                        }
+                }
+                
+                p_input->p->input.seeksegments[i_int]->eventNum = tempId;
+
+
+                vlc_mutex_unlock( &p_input->p->p_item->lock );
+                return VLC_SUCCESS;
+            }
+
+            vlc_mutex_unlock( &p_input->p->p_item->lock );
+            return VLC_EGENERIC;
+        }
+
+        case INPUT_REPLACE_SEGMENT_AT_IDX:  /* arg1=seeksegment_t*, arg2=int Replaces event in index arg2 with event arg1. */
+        {
+            p_segment = (seeksegment_t *)va_arg( args, seeksegment_t * );
+            i_int = (int)va_arg( args, int );
+
+            vlc_mutex_lock( &p_input->p->p_item->lock );
+
+            if ( p_segment && ( i_int >= 0 ) && ( i_int < p_input->p->input.i_seeksegments ) )
+            {
+                vlc_seeksegment_Delete( p_input->p->input.seeksegments[i_int] );
+                p_input->p->input.seeksegments[i_int] = vlc_seeksegment_Duplicate( p_segment );
+
+                vlc_mutex_unlock( &p_input->p->p_item->lock );
+                return VLC_SUCCESS;
+            }
+
+            vlc_mutex_unlock( &p_input->p->p_item->lock );
+            return VLC_EGENERIC;
+        }
+
+        case INPUT_DELETE_SEGMENT_AT_IDX:   /* arg1=int, arg2=int. Deletes event in index arg1, pushes all events backward (if arg2 is not -1, pushes all event numbers inthe arg2 layer.*/
+        {
+            i_int = (int)va_arg( args, int );
+            int layerId = (int)va_arg( args, int );
+
+            vlc_mutex_lock( &p_input->p->p_item->lock );
+
+
+            if ( ( i_int >= 0 ) && ( i_int < p_input->p->input.i_seeksegments ) )
+            {
+                seeksegment_t* toDelete = p_input->p->input.seeksegments[i_int];
+                vlc_seeksegment_Delete( toDelete );
+
+                for ( int i = i_int; i < p_input->p->input.i_seeksegments - 1; i++ )    //Start moving segments back
+                {
+                    p_input->p->input.seeksegments[i] = p_input->p->input.seeksegments[i + 1];
+                    
+                    if ( p_input->p->input.seeksegments[i]->i_layer_id == layerId )     //Pushes back events numbers
+                        p_input->p->input.seeksegments[i]->eventNum--;
+                }
+
+                p_input->p->input.i_seeksegments--;
+
+                p_input->p->input.seeksegments = realloc( p_input->p->input.seeksegments,
+                              sizeof(seeksegment_t) * (p_input->p->input.i_seeksegments) ) ;
+
+                vlc_mutex_unlock( &p_input->p->p_item->lock );
+                return VLC_SUCCESS;
+            }
+
+            vlc_mutex_unlock( &p_input->p->p_item->lock );
+            return VLC_EGENERIC;
+        }
+        case INPUT_GET_SEGMENTS:    /* arg1=seeksegment_t***, arg2=int*, arg3=char*, res=can fail , if arg3 is null return all layers */
+        {
+            p_segments = (seeksegment_t ***)va_arg( args, seeksegment_t *** );
+            pi_seeksegments = (int*)va_arg( args, int * );
+            char *psz_layerName = (char *)va_arg( args, char * );
+
+            vlc_mutex_lock( &p_input->p->p_item->lock );
+
+            *pi_seeksegments = 0;
+            *p_segments = NULL;
+
+            if( p_input->p->input.i_seeksegments > 0 )
+            {
+                if (!psz_layerName) /* Return segments for all events layers */
+                {
+                    *pi_seeksegments = p_input->p->input.i_seeksegments;
+                    *p_segments = malloc( sizeof(seeksegment_t *) *
+                                        p_input->p->input.i_seeksegments );
+
+                    for( int i = 0; i < p_input->p->input.i_seeksegments; i++ )
+                    {
+                        (*p_segments)[i] =
+                            vlc_seeksegment_Duplicate( p_input->p->input.seeksegments[i] );
+                    }
+                }
+                else    /* Only return segments for the requested events layer */
+                {
+                    for( int i = 0; i < p_input->p->input.i_seeksegments; i++ )
+                    {
+                        if ( strnicmp( p_input->p->input.seeksegments[i]->psz_layerName, psz_layerName, 255 ) != 0 ) /* Wrong layer event */
+                            continue;
+
+                        (*pi_seeksegments)++;
+                        *p_segments = realloc( *p_segments, (*pi_seeksegments) * sizeof(seeksegment_t *) );
+
+                        (*p_segments)[(*pi_seeksegments)-1] =
+                            vlc_seeksegment_Duplicate( p_input->p->input.seeksegments[i] );
+                    }
+                }
+
+                vlc_mutex_unlock( &p_input->p->p_item->lock );
+                return VLC_SUCCESS;
+            }
+            else
+            {
+                *p_segments = NULL;
+                *pi_seeksegments = -1;
+
+                vlc_mutex_unlock( &p_input->p->p_item->lock );
+                return VLC_EGENERIC;
+            }
+        }
+        case INPUT_GET_SELECTED_SEGMENTS:    /* arg1=seeksegment_t***, arg2=int*, res=can fail */
+        {
+            p_segments = (seeksegment_t ***)va_arg( args, seeksegment_t *** );
+            pi_seeksegments = (int*)va_arg( args, int * );
+
+            *pi_seeksegments = 0;
+            *p_segments = NULL;
+
+            vlc_mutex_lock( &p_input->p->p_item->lock );
+
+            for( int i = 0; i < p_input->p->input.i_seeksegments; i++ )
+            {
+                for ( int j = 0; j < p_input->p->input.i_selected ; j++ )   /* check selection for each layer name */
+                {
+                  int i_layer = p_input->p->input.pi_selectedIndex[j];   
+
+                  if ( strnicmp( p_input->p->input.seeksegments[i]->psz_layerName, p_input->p->input.psz_layerNames[i_layer], 255 ) == 0 )
+                  {
+                       (*pi_seeksegments)++;
+                       *p_segments = realloc( *p_segments, (*pi_seeksegments) * sizeof(seeksegment_t *) );
+                       
+                       (*p_segments)[(*pi_seeksegments)-1] =
+                            vlc_seeksegment_Duplicate( p_input->p->input.seeksegments[i] );
+                  }
+
+                }                
+            }
+            
+            vlc_mutex_unlock( &p_input->p->p_item->lock );
+            return VLC_SUCCESS;
+
+        }
+        case INPUT_GET_ANNOTATIONS:    /* arg1=seeksegment_t** returned event, int* event_index, int64_t queryTime, int layerId ; res=can fail, get annotation segment(s), 
+                                       if arg3 is -1 use current time, return segment if it matches arg4 layer, if arg4 is -1 return only if in selected segments*/
+        {
+            p_segments = (seeksegment_t **)va_arg( args, seeksegment_t ** );
+            pi_int = (int*)va_arg( args, int * );
+            i_64 = (int64_t)va_arg( args, int64_t );
+            int i_idx = (int)va_arg( args, int );
+
+            vlc_mutex_lock( &p_input->p->p_item->lock );
+            *p_segments = NULL;
+
+            if ( pi_int )
+                *pi_int = -1;
+
+            if( p_input->p->input.i_seeksegments > 0 )
+            {
+                int64_t runTime;
+                if ( i_64 == -1 )   /* No time mentioned - get current annotations */
+                    runTime = var_GetTime( p_input, "time" );
+                else 
+                    runTime = i_64;
+    
+                for( int i = 0; i < p_input->p->input.i_seeksegments; i++ )
+                {
+                    int64_t startTime = p_input->p->input.seeksegments[i]->i_time_offset;
+                    int64_t durTime = p_input->p->input.seeksegments[i]->i_time_duration;
+
+                    if ( ( startTime <= runTime ) && ( startTime + durTime >= runTime ) )   //In annotation range
+                    {
+                        if ( i_idx == -1 ) //only if is in selected segments
+                        {
+                            for ( int j = 0 ; j < p_input->p->input.i_selected ; j++ )
+                                if ( p_input->p->input.seeksegments[i]->i_layer_id == p_input->p->input.pi_selectedIndex[j] )
+                                {
+                                    *p_segments = vlc_seeksegment_Duplicate( p_input->p->input.seeksegments[i] );
+                                    
+                                    if ( pi_int )
+                                        *pi_int = i;
+
+                                    break;
+                                }
+                        }
+                        else if ( p_input->p->input.seeksegments[i]->i_layer_id == i_idx )
+                        {
+                                *p_segments = vlc_seeksegment_Duplicate( p_input->p->input.seeksegments[i] );
+                                
+                                if ( pi_int )
+                                    *pi_int = i;
+
+                                break;
+                        }
+                        //ROIY - modify for multiple events returned     
+                    }
+
+                    if ( startTime > runTime )   //Already past current point
+                        break;
+                }           
+            }
+
+            vlc_mutex_unlock( &p_input->p->p_item->lock );
+            return VLC_SUCCESS;
+        }
+
+        case INPUT_GET_NEXT_SEGMENT_IDX:    /* arg1=int*, arg2=int64_t. returns active event index at input time (current time if arg2 = -1) */
+            
+            pi_int = (int*)va_arg( args, int* );
+            i_64 = (int64_t)va_arg( args, int64_t );
+
+            if ( p_input )
+            {
+                int64_t queryTime;
+                if ( i_64 == -1 )   /* No time mentioned - use current time */
+                    queryTime = var_GetTime( p_input, "time" );
+                else 
+                    queryTime = i_64;
+    
+                vlc_mutex_lock( &p_input->p->p_item->lock );
+
+                int i;
+                for( i = 0; i < p_input->p->input.i_seeksegments; i++ )
+                {
+                    int64_t startTime = p_input->p->input.seeksegments[i]->i_time_offset;
+
+                    if ( queryTime < startTime )    //Found location
+                        break;
+                }
+
+                vlc_mutex_unlock( &p_input->p->p_item->lock );
+                *pi_int = i;
+                return VLC_SUCCESS;
+            }
+
+            return VLC_EGENERIC;
+
+        case INPUT_GET_NEXT_SELECTED_SEGMENT:    /* arg1=seeksegment_t**, res=can fail, get next selected annotation segment */
+            p_segments = (seeksegment_t **)va_arg( args, seeksegment_t ** );
+
+            vlc_mutex_lock( &p_input->p->p_item->lock );
+            *p_segments = NULL;
+
+            if( p_input->p->input.i_selected > 0 )
+            {
+                int64_t runTime = var_GetTime( p_input, "time" );
+    
+                for( int i = 0; i < p_input->p->input.i_seeksegments; i++ )
+                {
+                    int64_t startTime = p_input->p->input.seeksegments[i]->i_time_offset;
+
+                    if ( runTime < startTime  )    //valid next segment, ROIY - fix nonlinear event problem, improve to quicker sort?
+                    {
+                        for ( int j = 0 ; j < p_input->p->input.i_selected ; j++ )
+                            if ( p_input->p->input.seeksegments[i]->i_layer_id == p_input->p->input.pi_selectedIndex[j] )
+                            {
+                                *p_segments = vlc_seeksegment_Duplicate( p_input->p->input.seeksegments[i] );
+                                vlc_mutex_unlock( &p_input->p->p_item->lock );
+                                return VLC_SUCCESS;
+                            }
+                    }       //ROIY - fix case of adding layer while after last event
+                }           
+            }
+
+            vlc_mutex_unlock( &p_input->p->p_item->lock );
+            return VLC_SUCCESS;
+
+        case INPUT_GET_PREV_SELECTED_SEGMENT:    /* arg1=seeksegment_t**, res=can fail, get previous selected annotation segment */
+            p_segments = (seeksegment_t **)va_arg( args, seeksegment_t ** );
+
+            vlc_mutex_lock( &p_input->p->p_item->lock );
+            *p_segments = NULL;
+
+            if( p_input->p->input.i_selected > 0 )
+            {
+                int64_t runTime = var_GetTime( p_input, "time" );
+    
+                for( int i = p_input->p->input.i_seeksegments - 1; i >= 0; i-- )
+                {
+                    int64_t startTime = p_input->p->input.seeksegments[i]->i_time_offset;
+                    int64_t durTime = p_input->p->input.seeksegments[i]->i_time_duration;
+                    
+                    if ( durTime > 2000000 )    //2 seconds window for allowing the user to go to previous segment instead of beginning of current
+                        startTime += 2000000;
+                    else                        //Always skip backwards in a very small ( under 2 secs ) segment
+                        startTime += durTime;
+
+                    for ( int j = 0 ; j < p_input->p->input.i_selected ; j++ )
+                        if ( p_input->p->input.seeksegments[i]->i_layer_id == p_input->p->input.pi_selectedIndex[j] )
+                            if ( runTime > startTime )      //Found
+                            {
+                                *p_segments = vlc_seeksegment_Duplicate( p_input->p->input.seeksegments[i] );
+                                vlc_mutex_unlock( &p_input->p->p_item->lock );
+                                return VLC_SUCCESS;
+                            }
+                }
+            }
+
+            vlc_mutex_unlock( &p_input->p->p_item->lock );
+            return VLC_SUCCESS;
+
+        case INPUT_GET_ANNOTATIONS_TIME:    
+        /* arg1=char*, arg2=int64_t, arg3=int64_t* res=can fail. Gets elapsed segments time of layer [arg1];
+           if arg1 is NULL get data for all events. To get total segments time, mark arg2 as -1. */
+        {
+            char *psz_layerName = (char *)va_arg( args, char * );
+            i_64 = (int64_t)va_arg( args, int64_t );
+            pi_64 = (int64_t*)va_arg( args, int64_t * );
+
+            vlc_mutex_lock( &p_input->p->p_item->lock );
+
+            int64_t startTime, durTime, timeCounter = 0;
+ 
+            for( int i = 0; i < p_input->p->input.i_seeksegments; i++ )
+            {
+                if ( psz_layerName )    //Only count for specific events layer
+                {
+                    if ( strnicmp( p_input->p->input.seeksegments[i]->psz_layerName, psz_layerName, 255 ) != 0 ) /* Wrong layer event */
+                        continue;
+                }
+                else
+                {       //Make sure it is a selected segment
+                    int j;
+                    for ( j = 0 ; j < p_input->p->input.i_selected ; j++ )
+                        if ( p_input->p->input.seeksegments[i]->i_layer_id == p_input->p->input.pi_selectedIndex[j] )
+                            break;
+                    if ( j == p_input->p->input.i_selected )
+                        continue;
+                }
+
+                startTime = p_input->p->input.seeksegments[i]->i_time_offset;
+                durTime = p_input->p->input.seeksegments[i]->i_time_duration;
+
+                if ( i_64 == -1 )   //Find total time, simply add
+                {
+                    timeCounter += durTime;
+                    continue;
+                }
+
+                else if ( i_64 < startTime )
+                {
+                    break;
+                }
+                
+                else if ( i_64 < startTime + durTime)
+                {
+                    timeCounter += ( i_64 - startTime );  //Add the time elapsed in current segment
+                    break;
+                }
+
+                else    //Add segment to counter
+                {
+                    timeCounter += durTime;
+                }
+            }           
+            
+
+            *pi_64 = timeCounter;
+
+            vlc_mutex_unlock( &p_input->p->p_item->lock );
+            return VLC_SUCCESS;
+        }
+        case INPUT_GET_LAYER_NAMES:    /* arg1=char****, arg2=int**, returns layers number and layers names array */
+            psz_array = (char****)va_arg( args, char**** );
+            ppi_int = (int**)va_arg( args, int** );
+
+            if ( p_input )
+            {
+                vlc_mutex_lock( &p_input->p->p_item->lock );
+
+                *psz_array = &(p_input->p->input).psz_layerNames;
+                *ppi_int = &(p_input->p->input).i_layers;
+
+                vlc_mutex_unlock( &p_input->p->p_item->lock );
+                return VLC_SUCCESS;
+            }
+
+            return VLC_EGENERIC;
+
+       case INPUT_GET_SELECTED_LAYERS:    /* arg1=int***, arg2=int**, returns selected layers index numbers and size */
+       { 
+            int ***pi_idx_array = (int***)va_arg( args, int*** );
+
+            int **ppi_int = (int**)va_arg( args, int** );
+
+            if ( p_input )
+            {
+                vlc_mutex_lock( &p_input->p->p_item->lock );
+                
+                *pi_idx_array = &(p_input->p->input).pi_selectedIndex;
+                *ppi_int = &(p_input->p->input).i_selected;
+                
+                vlc_mutex_unlock( &p_input->p->p_item->lock );
+                return VLC_SUCCESS;
+            }
+
+            return VLC_EGENERIC;
+       }
+        case INPUT_SELECT_LAYER:    
+        /* arg1=int, res=can fail. Selects and adds event layer [arg1] to player;
+           if arg1 is -1 add all layers */
+        {
+            int i_idx = (int)va_arg( args, int );
+
+            vlc_mutex_lock( &p_input->p->p_item->lock );
+
+            for ( int i = 0; i < p_input->p->input.i_selected ; i++ )
+            {
+                if ( p_input->p->input.pi_selectedIndex[i] == i_idx )   /* Layer ID already selected */
+                {
+                    vlc_mutex_unlock( &p_input->p->p_item->lock );
+                    return VLC_EGENERIC;
+                }
+            }
+
+            p_input->p->input.pi_selectedIndex = realloc( p_input->p->input.pi_selectedIndex, (p_input->p->input.i_selected+1) * sizeof(int) );           
+            if ( !p_input->p->input.pi_selectedIndex )
+            {
+                vlc_mutex_unlock( &p_input->p->p_item->lock );
+                return VLC_EGENERIC;
+            }
+            
+            p_input->p->input.i_selected++;     /* Increment number of selected layers */
+            p_input->p->input.pi_selectedIndex[p_input->p->input.i_selected-1] = i_idx;
+
+            vlc_mutex_unlock( &p_input->p->p_item->lock );
+            return VLC_SUCCESS;
+        }
+        case INPUT_DESELECT_LAYER:    
+        /* arg1=int, res=can fail. Deselects and removes event layer [arg1] from player;
+           if arg1 is -1 remove all layers */
+        {
+            int i_idx = (int)va_arg( args, int );
+
+            vlc_mutex_lock( &p_input->p->p_item->lock );  
+
+            for ( int i = 0; i < p_input->p->input.i_selected ; i++ )
+            {
+                if ( p_input->p->input.pi_selectedIndex[i] == i_idx )   /*  Found layer */
+                {
+                    p_input->p->input.pi_selectedIndex[i] = p_input->p->input.pi_selectedIndex[p_input->p->input.i_selected-1];  /* move last item to empty location */
+                    p_input->p->input.i_selected--;     /* Decrement number of selected layers */
+
+                    p_input->p->input.pi_selectedIndex = realloc( p_input->p->input.pi_selectedIndex, p_input->p->input.i_selected * sizeof(int) );
+
+                    vlc_mutex_unlock( &p_input->p->p_item->lock );
+
+                    if ( !p_input->p->input.pi_selectedIndex )
+                        return VLC_EGENERIC;
+
+                    return VLC_SUCCESS;
+                }
+            }
+
+            vlc_mutex_unlock( &p_input->p->p_item->lock );
+            return VLC_EGENERIC;
+        }
+        case INPUT_ADD_LAYER:  /* arg1=char** */
+        {
+            char** layerName = (char**)va_arg( args, char** );
+
+            vlc_mutex_lock( &p_input->p->p_item->lock ); 
+
+            p_input->p->input.i_layers++;
+            p_input->p->input.psz_layerNames = realloc( p_input->p->input.psz_layerNames, p_input->p->input.i_layers * sizeof(char*) );   //Add layer to array
+            if ( !p_input->p->input.psz_layerNames ) 
+            {
+                vlc_mutex_unlock( &p_input->p->p_item->lock );
+
+                return VLC_EGENERIC;
+            }
+            if ( layerName )
+                p_input->p->input.psz_layerNames[p_input->p->input.i_layers-1] = *layerName;
+            else
+                p_input->p->input.psz_layerNames[p_input->p->input.i_layers-1] = NULL;
+
+            vlc_mutex_unlock( &p_input->p->p_item->lock );
+
+            return VLC_SUCCESS;
+        }
+        case INPUT_SAVE_ENHANCEMENT_FILE:    /* arg1=char*. Saves enhancement data file to arg1 */
+            psz = (char*)va_arg( args, char* );
+            //ROIY USE psz from dialog, null usedeafult
+            vlc_mutex_lock( &p_input->p->p_item->lock );  
+
+            /* make sure there is med type slave */
+            int i = getSlaveId( p_input, "med" );
+
+            if ( i == -1 )     //no med slave -> med file did not exist, initialize empty file, create med slave
+            {
+                if ( initEnhancementFile( p_input ) == VLC_SUCCESS )
+                {
+                    input_ControlPush( p_input, INPUT_CONTROL_LOAD_ENHANCEMENT_DATA, &val );
+                    if ( val.b_bool )
+                    {
+                        vlc_mutex_unlock( &p_input->p->p_item->lock );
+                        return VLC_SUCCESS;
+                    }
+                }
+
+                vlc_mutex_unlock( &p_input->p->p_item->lock );
+                return VLC_EGENERIC;
+            }
+
+            /* Tell slave to save */
+            if( demux_Control( p_input->p->slave[i]->p_demux, DEMUX_SAVE_FILE,
+                            p_input->p->p_item->psz_uri ) != VLC_SUCCESS )
+            {
+                msg_Err( p_input, "Could not save data to file", p_input->p->p_item->psz_uri );
+                vlc_mutex_unlock( &p_input->p->p_item->lock );
+                return VLC_EGENERIC;
+            }
+
+            vlc_mutex_unlock( &p_input->p->p_item->lock );
+            return VLC_SUCCESS;
+
         case INPUT_CLEAR_BOOKMARKS:
 
             vlc_mutex_lock( &p_input->p->p_item->lock );
@@ -375,6 +1021,19 @@ int input_vaControl( input_thread_t *p_input, int i_query, va_list args )
             }
             return VLC_SUCCESS;
 
+        case INPUT_CHECK_SLAVE_EXISTS:
+            psz = (char*)va_arg( args, char * );
+            bool* b_exists = (bool*)va_arg( args, bool* );
+            if( psz )
+            {
+                int i = getSlaveId( p_input, psz );
+                if ( i == -1 )
+                    *b_exists = false;
+                else
+                    *b_exists = true;
+            }
+            return VLC_SUCCESS;
+
         case INPUT_ADD_SUBTITLE:
             psz = (char*)va_arg( args, char * );
             b_bool = (bool)va_arg( args, int );
@@ -555,3 +1214,40 @@ static void UpdateBookmarksOption( input_thread_t *p_input )
     input_SendEventBookmark( p_input );
 }
 
+/* ROIY - move this somewhere? */ 
+static int initEnhancementFile( input_thread_t *p_input )
+{
+    char* filename = strdup( p_input->p->p_item->psz_uri );
+
+    filename = make_path( filename );
+
+    /* Replace extension with .med */
+    char* dot = strrchr( filename, '.' );
+    *dot  = '\0';   
+    strcat( filename, ".med" );
+
+    FILE *pFile = vlc_fopen( filename, "wt" );
+
+    if( pFile == NULL )
+    {
+        msg_Err( p_input, "could not create .med file %s (%m)",  filename );
+        return VLC_EGENERIC;
+    }
+
+    fprintf(pFile, "med\n\n\nmed");
+    fclose(pFile);
+
+    return VLC_SUCCESS;
+}
+
+static int getSlaveId( input_thread_t *p_input, const char* psz_slave )
+{
+    for ( int i = 0 ; i < p_input->p->i_slave ; i++ )
+    {
+        if ( !strcmp( p_input->p->slave[i]->p_demux->psz_demux , psz_slave ) )
+            return i;  //found slave
+    }
+
+    return -1;
+}
+
diff --git a/src/input/event.c b/src/input/event.c
index 453e04b..71c37fc 100644
--- a/src/input/event.c
+++ b/src/input/event.c
@@ -162,6 +162,11 @@ void input_SendEventSeekpoint( input_thread_t *p_input, int i_title, int i_seekp
     Trigger( p_input, INPUT_EVENT_CHAPTER );
 }
 
+void input_SendEventSeeksegments( input_thread_t *p_input)
+{
+    Trigger( p_input, INPUT_EVENT_SEEKSEGMENTS );    //med events updated
+}
+
 void input_SendEventSignal( input_thread_t *p_input, double f_quality, double f_strength )
 {
     vlc_value_t val;
diff --git a/src/input/event.h b/src/input/event.h
index c8f1071..2b384dc 100644
--- a/src/input/event.h
+++ b/src/input/event.h
@@ -40,6 +40,7 @@ void input_SendEventSubtitleDelay( input_thread_t *p_input, mtime_t i_delay );
 void input_SendEventRecord( input_thread_t *p_input, bool b_recording );
 void input_SendEventTitle( input_thread_t *p_input, int i_title );
 void input_SendEventSeekpoint( input_thread_t *p_input, int i_title, int i_seekpoint );
+void input_SendEventSeeksegments( input_thread_t *p_input);
 void input_SendEventSignal( input_thread_t *p_input, double f_quality, double f_strength );
 void input_SendEventState( input_thread_t *p_input, int i_state );
 void input_SendEventCache( input_thread_t *p_input, double f_level );
diff --git a/src/input/input.c b/src/input/input.c
index 1acf976..d9adf50 100644
--- a/src/input/input.c
+++ b/src/input/input.c
@@ -351,6 +351,12 @@ static input_thread_t *Create( vlc_object_t *p_parent, input_item_t *p_item,
     p_input->p->input.i_title  = 0;
     p_input->p->input.title    = NULL;
     p_input->p->input.i_title_offset = p_input->p->input.i_seekpoint_offset = 0;
+    p_input->p->input.i_seeksegments = 0;   //ROIY-FIX?
+    p_input->p->input.seeksegments = NULL;  //ROIY-FIX?
+    p_input->p->input.i_layers = 0;
+    p_input->p->input.psz_layerNames = NULL;
+    p_input->p->input.i_selected = 0;
+    p_input->p->input.pi_selectedIndex = NULL;
     p_input->p->input.b_can_pace_control = true;
     p_input->p->input.b_can_rate_control = true;
     p_input->p->input.b_rescale_ts = true;
@@ -776,7 +782,7 @@ static void MainLoop( input_thread_t *p_input, bool b_interactive )
                         continue;
                     break;
                 }
-
+                
 #ifndef NDEBUG
                 msg_Dbg( p_input, "control type=%d", i_type );
 #endif
@@ -788,7 +794,6 @@ static void MainLoop( input_thread_t *p_input, bool b_interactive )
                     b_force_update = true;
                 }
             }
-
             /* Update interface and statistics */
             i_current = mdate();
             if( i_intf_update < i_current || b_force_update )
@@ -1049,6 +1054,44 @@ static void LoadSubtitles( input_thread_t *p_input )
         var_Destroy( p_input, "sub-description" );
 }
 
+static int LoadEnhancementData( input_thread_t *p_input )
+{
+    input_source_t *med_source;
+    vlc_value_t count;
+    vlc_value_t list;
+
+    //bool featureEnabled = true;   //ROIY - add feature enable option in gui
+
+    msg_Dbg( p_input, "Loading Enhancement Data");
+
+    /* Load Enhancement Data (Event Layers) */
+       
+
+    char *psz_autopath = "";
+    char **ppsz_med_files = med_Detect( p_input, psz_autopath,
+                                         p_input->p->p_item->psz_uri );
+                                                     
+    char *url = vlc_path2uri( ppsz_med_files[0], NULL );  //ROIY - TODO, parse all files?
+                                                     
+    var_Change( p_input, "spu-es", VLC_VAR_CHOICESCOUNT, &count, NULL );
+                                                     
+    med_source = InputSourceNew( p_input );                 
+    if( !med_source || !url
+     || InputSourceInit( p_input, med_source, url, "med", (SUB_CANFAIL) ) )  //ROIY - fix to: Feature disabled and no med file, or failed to init
+    {
+        free( med_source );
+        free( url );
+        msg_Err( p_input, "Failed to load");
+        return VLC_EGENERIC;
+    }
+    free( url );
+    TAB_APPEND( p_input->p->i_slave, p_input->p->slave, med_source );
+    input_SendEventSeeksegments(p_input);    //Signal events update for slider , ROIY -fix?
+
+    msg_Dbg( p_input, "Finished loading Enhancement Data successfully");
+    return VLC_SUCCESS;
+}
+
 static void LoadSlaves( input_thread_t *p_input )
 {
     char *psz = var_GetNonEmptyString( p_input, "input-slave" );
@@ -1229,6 +1272,7 @@ static int Init( input_thread_t * p_input )
     {
         StartTitle( p_input );
         LoadSubtitles( p_input );
+        LoadEnhancementData( p_input );  //Try to get Enhancement data
         LoadSlaves( p_input );
         InitPrograms( p_input );
 
@@ -1949,6 +1993,13 @@ static bool Control( input_thread_t *p_input,
                 SubtitleAdd( p_input, val.psz_string, true );
             break;
 
+        case INPUT_CONTROL_LOAD_ENHANCEMENT_DATA:
+            if ( LoadEnhancementData( p_input ) == VLC_SUCCESS )
+                val.b_bool = true;
+            else 
+                val.b_bool = false;
+            break;
+
         case INPUT_CONTROL_ADD_SLAVE:
             if( val.psz_string )
             {
@@ -2382,7 +2433,6 @@ static int InputSourceInit( input_thread_t *p_input,
             msg_Warn( p_input, "cannot create a stream_t from access" );
             goto error;
         }
-
         /* Add stream filters */
         char *psz_stream_filter = var_GetNonEmptyString( p_input,
                                                          "stream-filter" );
@@ -2465,7 +2515,7 @@ static int InputSourceInit( input_thread_t *p_input,
 #else
     var_SetBool( p_input, "can-record", in->b_can_stream_record );
 #endif
-
+    
     /* get attachment
      * FIXME improve for b_preparsing: move it after GET_META and check psz_arturl */
     if( !p_input->b_preparsing )
@@ -2531,6 +2581,25 @@ static void InputSourceClean( input_source_t *in )
             vlc_input_title_Delete( in->title[i] );
         TAB_CLEAN( in->i_title, in->title );
     }
+
+    for( i = 0; i < in->i_seeksegments; i++ )
+    {
+        free( in->seeksegments[i]->psz_layerName );
+        free( in->seeksegments[i]->psz_text );
+        free( in->seeksegments[i] );
+    }
+    free( in->seeksegments );
+    in->i_seeksegments = 0;
+
+
+    for( i = 0; i < in->i_layers; i++ )
+        free( in->psz_layerNames[i] );
+
+    free( in->psz_layerNames );
+    in->i_layers = 0;
+
+    free( in->pi_selectedIndex );
+    in->i_selected = 0;
 }
 
 /*****************************************************************************
diff --git a/src/input/input_internal.h b/src/input/input_internal.h
index 9f71294..9f9a5be 100644
--- a/src/input/input_internal.h
+++ b/src/input/input_internal.h
@@ -54,6 +54,16 @@ typedef struct
     int i_seekpoint_start;
     int i_seekpoint_end;
 
+    /* Title seeksegments */
+    int         i_seeksegments;
+    seeksegment_t **seeksegments;
+    
+    /* Events Layers */
+    int         i_layers;
+    char        **psz_layerNames;
+    int         i_selected;
+    int         *pi_selectedIndex;
+
     /* Properties */
     bool b_can_pause;
     bool b_can_pace_control;
@@ -211,6 +221,8 @@ enum input_control_e
     INPUT_CONTROL_SET_RECORD_STATE,
 
     INPUT_CONTROL_SET_FRAME_NEXT,
+
+    INPUT_CONTROL_LOAD_ENHANCEMENT_DATA
 };
 
 /* Internal helpers */
@@ -246,6 +258,10 @@ void input_ConfigVarInit ( input_thread_t * );
 char **subtitles_Detect( input_thread_t *, char* path, const char *fname );
 int subtitles_Filter( const char *);
 
+/* med events */
+char **med_Detect( input_thread_t *p_this, char *psz_path, const char *psz_name_org );
+int med_Filter( const char *psz_dir_content );
+
 /* input.c */
 void input_SplitMRL( const char **, const char **, const char **,
                      const char **, char * );
diff --git a/src/input/var.c b/src/input/var.c
index 6ec28f0..3afcb8b 100644
--- a/src/input/var.c
+++ b/src/input/var.c
@@ -220,6 +220,8 @@ void input_ControlVarInit ( input_thread_t *p_input )
     var_Create( p_input, "bit-rate", VLC_VAR_INTEGER );
     var_Create( p_input, "sample-rate", VLC_VAR_INTEGER );
 
+    var_Create( p_input, "autoJump", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );    //ROIY - needed?
+
     if( !p_input->b_preparsing )
     {
         /* Special "intf-event" variable. */
diff --git a/src/playlist/engine.c b/src/playlist/engine.c
index a80c3ef..54f703f 100644
--- a/src/playlist/engine.c
+++ b/src/playlist/engine.c
@@ -491,6 +491,7 @@ static void VariablesInit( playlist_t *p_playlist )
     var_Create( p_playlist, "loop", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
     var_Create( p_playlist, "corks", VLC_VAR_INTEGER );
     var_AddCallback( p_playlist, "corks", CorksCallback, NULL );
+    var_Create( p_playlist, "autoJump", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
 
     var_Create( p_playlist, "rate", VLC_VAR_FLOAT | VLC_VAR_DOINHERIT );
     var_AddCallback( p_playlist, "rate", RateCallback, NULL );
-- 
1.8.3.msysgit.0

